<?php

namespace App\Http\Controllers;

use Illuminate\Support\Facades\Log;
use Validator;
use Illuminate\Support\Facades\Mail;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Session;
use App\Mail\UserFortgotMail;
use Illuminate\Support\Facades\DB;
use App\Models\tbl_page;
use App\Models\tbl_banner;
use App\Models\location;
use App\Models\partners;
use App\Models\customers;
use App\Models\websiteMange;

class CustomerController extends Controller
{
 
public function CustomerRegisterFrom(Request $request)
{
    try {
        $request->validate([
            'name' => 'required|unique:customers,name',
            'email' => 'required|email|unique:customers,email',
            'password' => 'required|min:6',
        ]);

        
        $lastRegisterId = customers::where('register_id', 'LIKE', 'FRZ-U%')
            ->selectRaw("CAST(SUBSTRING(register_id, 6) AS UNSIGNED) AS id_number")
            ->orderByDesc('id_number')
            ->value('id_number');

        $newIdNumber = $lastRegisterId ? $lastRegisterId + 1 : 1;
        $newRegisterId = 'FRZ-U' . str_pad($newIdNumber, 2, '0', STR_PAD_LEFT);

        
        $customer = customers::create([
            'name' => $request->name,
            'email' => $request->email,
            'phone' => $request->phone,
            'register_id' => $newRegisterId,
            'password' => bcrypt($request->password),
            'status' => 'pending', 
        ]);

        
        $otp = rand(100000, 999999);
        Session::put('otp', $otp); 
        Session::put('customer_id', $customer->id); 

        Mail::to($request->email)->send(new \App\Mail\SendOtpMail($otp));

        return response()->json([
            'message' => 'Registration successful. Verify your email to proceed.',
            'status' => 'otp_sent',
        ]);
    } catch (\Exception $e) {
        return response()->json([
            'message' => 'An error occurred: ' . $e->getMessage(),
            'status' => 'error',
        ]);
    }
}


public function verifyOtp(Request $request)
{
    $request->validate([
        'otp' => 'required|digits:6',
    ]);

    if ($request->otp == session('otp')) {
        $customer = customers::find(session('customer_id'));
        if (!$customer) {
            return response()->json([
                'success' => false,
                'message' => 'Customer not found.',
            ]);
        }

        $customer->status = 'active';
        $customer->save();

       
        session([
            'id' => $customer->id,
            'name' => strtolower(str_replace(' ', '-', $customer->name)),
            'register_id' => $customer->register_id,
        ]);

        
        return response()->json([
            'success' => true,
            'message' => 'Email verified successfully!',
            'redirect_url' => route('profile', [
                'customername' => strtolower(str_replace(' ', '-', $customer->name)),
                'register_id' => $customer->register_id,
            ]),
        ]);
    } else {
        return response()->json([
            'success' => false,
            'message' => 'Invalid OTP. Please try again.',
        ]);
    }
}






public function CustomerLogin(Request $request)
{
    
    $request->validate([
        'name' => 'required',
        'password' => 'required',
    ]);

   
    $credentials = $request->only('name', 'password');

   
    $partner = customers::where('name', $credentials['name'])->first();

    
    if ($partner && Hash::check($credentials['password'], $partner->password) && $partner->status === 'active') {
    
        session([
            'id' => $partner->id,
            'customers' => $partner->customers,
            'register_id' => $partner->register_id,
            'name' => $partner->name,
        ]);

       
        return response()->json([
            'success' => true,
            'message' => 'Welcome!',
            'redirect_url' => route('profile', [
                'customername' => $partner->name,
                'register_id' => $partner->register_id,
            ]),
        ]);
    }

    
    return response()->json([
        'success' => false,
        'message' => $partner && Hash::check($credentials['password'], $partner->password) 
                     ? 'Account is not active.' 
                     : 'Invalid login credentials.',
    ]);
}



public function customerLogout()
{
    session()->forget(['id', 'customername', 'register_id']);
    

    return redirect()->route('home')->withSuccess('Logged out successfully.');
}




public function CustomerDataUpdte(Request $request)
{
    try {
        $validatedData = $request->validate([
            'profile_image' => 'nullable|image|max:2048',
            'name' => 'required|string|max:255',
            'email' => 'required|email|max:255',
            'phone' => 'required|string|max:15',
            'address' => 'nullable|string|max:500',
        ]);

        $existingCustomer = customers::where('register_id', $request->register_id)->first();

        if (!$existingCustomer) {
            return response()->json(['success' => false, 'message' => 'Customer not found.']);
        }

        $profile_image = $existingCustomer->profile_image ?? null;
        if ($request->hasFile('profile_image')) {
            $profile_image = rand() . '.' . $request->file('profile_image')->getClientOriginalExtension();
            $request->file('profile_image')->move(public_path('/images'), $profile_image);
        }

        $existingCustomer->update([
            'profile_image' => $profile_image,
            'name' => $request->name,
            'email' => $request->email,
            'phone' => $request->phone,
            'address' => $request->address,
        ]);

        return response()->json(['success' => true, 'message' => 'Profile updated successfully!']);
    } catch (\Exception $e) {
        return response()->json(['success' => false, 'message' => 'An error occurred: ' . $e->getMessage()]);
    }
}


public function UserForgotPass(Request $request) {
    $homepages = tbl_page::where('pg_status', 'active')->where('pg_url', 'forgot-pass')->get();
    
    $website = websiteMange::where('status', 'active')->get();
   
    return view('user-foreget-pass', ['homepages' => $homepages, 'website' => $website]);
}


public function ForgotPass(Request $request)
{
    
    if ($request->step === 'request_otp') {
        $request->validate(['email' => 'required|email']);
        
        
        $customer = customers::where('email', trim($request->email))->first();

        if (!$customer) {
            return response()->json(['success' => false, 'message' => 'Email not found.'], 404);
        }

        $otp = rand(100000, 999999);
DB::table('customers')
    ->where('email', $customer->email)
    ->update(['otp' => $otp]);

Log::info('Generated OTP: ' . $otp . ' for email: ' . $customer->email);



        
        try {
            Mail::to($customer->email)->send(new UserFortgotMail($otp));
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => 'Failed to send OTP. Please try again.'], 500);
        }

        return response()->json(['success' => true, 'message' => 'OTP sent to your email.']);
    }

    
    elseif ($request->step === 'verify_otp') {
        $request->validate(['email' => 'required|email', 'otp' => 'required']);

        
       $customer = customers::where('email', trim($request->email))
                      ->where('otp', trim($request->otp))
                      ->first();

            if (!$customer) {
                return response()->json([
                    'success' => false, 
                    'message' => 'Invalid OTP.',
                    'debug' => ['email' => trim($request->email), 'otp' => trim($request->otp)]
                ], 400);
            }

       
        $customer->update(['otp' => null]);

        return response()->json(['success' => true, 'message' => 'OTP verified.']);
    }

   
    elseif ($request->step === 'reset_password') {
        $request->validate([
            'email' => 'required|email',
            'password' => 'required|confirmed|min:6',
        ]);

        $customer = customers::where('email', $request->email)->first();
        
        if (!$customer) {
            return response()->json(['success' => false, 'message' => 'Invalid email.'], 404);
        }

         DB::table('customers')
    ->where('email', $customer->email)
    ->update(['password' => bcrypt($request->password), 'otp' => null]);
    
        

        return response()->json(['success' => true, 'message' => 'Password reset successfully.']);
    }

    return response()->json(['success' => false, 'message' => 'Invalid request.'], 400);
}




}
